#!/usr/bin/python

#
# Copyright 2001 Stfane Fermigier and Nuxeo SARL
# See LICENSE.TXT for licensing information
#
import re, string, unittest
import Zope, Acquisition
from Testing import makerequest
from Products.NuxDocument import NuxDocument

# Try CMF test. OK if they fail if CMF is not installed
try:
    from Products.CMFCore.tests.base.testcase import SecurityRequestTest
    from Products.CMFCore.TypesTool import FactoryTypeInformation
    from Products.NuxDocument import CMFNuxDocument
except:
    class SecurityTest: pass

TESTDATA = ''


class BaseTestCase(unittest.TestCase):
    def setUp(self):
        self.app = makerequest.makerequest(Zope.app())
        get_transaction().begin()

        # Catalog for CatalogAwareness testing
        self.app.manage_addProduct['ZCatalog']\
            .manage_addZCatalog('CatalogForTests', title='')
        self.catalog = self.app.CatalogForTests

    def open(self, filename):
        return open(TESTDATA + filename)

    def tearDown(self):
        get_transaction().abort()
        self.app._p_jar.close()


class DumbTestCase(BaseTestCase):
  def test_Dumb(self):
    title = 'XXX title YYY'
    data = 'qwreytroiquwe'
    self.app.manage_addProduct['NuxDocument']\
      .addZNuxDocument(id='doc', file=data, title=title)

    doc = self.app.doc
    self.assertEquals(doc.title, title)
    self.assertEquals(doc.getRaw(), data)

    self.assertEquals(doc.MimeType(), 'text/plain') # XXX: fix this


class MSWordTestCase(BaseTestCase):
  def setUp(self):
    BaseTestCase.setUp(self)
    self.data = self.open('test1.doc').read()
    self.doc = NuxDocument.NuxDocument(id='doc', file=self.data)

  def test1(self):
    pass


class ZMSWordTestCase(BaseTestCase):
  def setUp(self):
    BaseTestCase.setUp(self)
    self.data = self.open('test1.doc').read()
    self.app.manage_addProduct['NuxDocument']\
      .addZNuxDocument(id='doc', file=self.data)
    self.doc = self.app.doc

  def test_catalogSupport(self):
    data = self.doc.PrincipiaSearchSource()
    self.assertEquals(string.count(data, 'Linux'), 1)

  def test_catalog(self):
    brain_list = self.catalog(PrincipiaSearchSource='linux')
    self.assertEqual(len(brain_list), 1)

    brain_list = self.catalog(PrincipiaSearchSource='macos')
    print brain_list
    self.assertEqual(len(brain_list), 0)

  def test_rawAccess(self):
    return
    data = self.doc.document_src()
    self.assertEqual(data, self.data)

    data = self.doc.manage_FTPget()
    self.assertEqual(data, self.data)

  def test_render(self):
    self.assertEquals(string.count(self.doc(), 'Linux'), 1)

    #data = self.doc.document_src(REQUEST, RESPONSE)
    #REQUEST, RESPONSE = self.app.REQUEST, self.app.REQUEST.RESPONSE
    #self.assertEqual(RESPONSE.getHeader('Content-Type'), self.doc.mime_type)


class ZMSWordImagesTestCase(BaseTestCase):
  def setUp(self):
    BaseTestCase.setUp(self)
    self.data = self.open('test4.doc').read()
    self.app.manage_addProduct['NuxDocument']\
      .addZNuxDocument(id='doc', file=self.data)
    self.doc = self.app.doc

  def test_image(self):
    img = self.doc['doc0.png']
    self.assertEquals(img.get_size(), 5690)


#
# CMFDocument tests
#
class CMFMSWordTestCase1(BaseTestCase):
  "Test CMFNuxDocument with a Word document, using direct objet instantiation"

  def setUp(self):
    BaseTestCase.setUp(self)
    self.data = self.open('test1.doc').read()
    self.doc = CMFNuxDocument.CMFNuxDocument(id='doc', file=self.data)

  def test_mime_type(self):
    self.assertEqual(self.doc.MimeType(), 'application/msword')

  def test1(self):
    data = self.open('test2.doc').read()
    self.doc.edit('', file=data)

  def test_CookedBody(self):
    assert not re.search('<body(?i)', self.doc.CookedBody())


class CMFMSWordTestCase2(SecurityRequestTest, BaseTestCase):
  "Test CMFNuxDocument with a Word document, using CMF factory"

  def setUp( self ):
    from Products.CMFCore.utils import getToolByName

    SecurityRequestTest.setUp(self)
    self.app = self.root

    self.app.manage_addProduct['CMFDefault']\
      .manage_addCMFSite(id='CMFForTests')
    self.cmf = self.app.CMFForTests

    #
    # Setup Types Tools to recognize our new class
    #
    tt = getToolByName(self.cmf, 'portal_types')
    tt.manage_addTypeInformation(
      add_meta_type=FactoryTypeInformation.meta_type,
      id='NuxDocument', typeinfo_name='NuxDocument: CMFNuxDocument')
    self.assert_('NuxDocument' in tt.listContentTypes())

    # 
    # Create document by invoking factory
    #
    self.cmf.invokeFactory('NuxDocument', id='doc', title='xxxtitleyyy')
    self.doc = self.cmf.doc
    self.data = self.open('test1.doc').read()
    self.doc._edit(title='xxxtitleyyy', file=self.data, mime_type='')

  def test_mime_type(self):
    self.assertEqual(self.doc.MimeType(), 'application/msword')

  def test_portalCatalog(self):
    words = map(lambda x: x[0], self.cmf.portal_catalog.Vocabulary.words())
    self.assert_('linux' in words)

